from typing import List, Optional
from pydantic import BaseModel


class DataModel(BaseModel):
    id: str
    start_date: Optional[str] = None
    end_date: Optional[str] = None
    metrics_list: list = []
    dimensions_list: list = []
    chartTitle: Optional[str] = None
    secondary_metrics: Optional[str] = None

    class Config:
        schema_extra = {
            "example": {
                "id": "13406065",
                "start_date": "2021-10-01",
                "end_date": "2021-10-31",
                "metrics_list": {"ga:users", "ga:sessions"},
                "dimensions_list": {"ga:deviceCategory"},
                "secondary_metrics": "ga:bounceRate",
            }
        }


class MCFModel(BaseModel):
    ids: str
    start_date: Optional[str] = None
    end_date: Optional[str] = None
    metrics: list = []
    dimensions: str

    class Config:
        schema_extra = {
            "example": {
                "ids": "13406065",
                "start_date": "2021-10-01",
                "end_date": "2021-10-31",
                "metrics": {
                    "mcf:firstInteractionConversions",
                    "mcf:lastInteractionConversions",
                },
                "dimensions": "mcf:medium",
            }
        }


class TestingModel(BaseModel):
    id: Optional[str]

    class Config:
        schema_extra = {
            "example": {
                "id": "13406065",
            }
        }


def ResponseModel(data, message):
    return {
        "data": [data],
        "code": 200,
        "message": message,
    }


def ErrorResponseModel(error, code, message):
    return {"error": error, "code": code, "message": message}



from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from whale_googleanalytics.server.routes.routes_gcp import router as routes_gcp

app = FastAPI()

origins = ["http://localhost:3000"]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Routes
app.include_router(routes_gcp, tags=["api"], prefix="/api")

# Get routes
@app.get("/", tags=["Root"])
async def read_root():
    return {"message": "Welcome to Google Analytics"}


def pytest():
    return app



"""Google Analytics data to CSV
# gcp_functions.py: Pull Google Analytics from Official API GA4 - @Credit: 4th Whale Marketing 2021
No header is written for the csv generated.
To be used with Airflow to import a Google Analytics data.
"""

from googleapiclient.discovery import build
from oauth2client.service_account import ServiceAccountCredentials

import pandas as pd
import numpy as np
from datetime import timedelta, datetime
from random import randrange, uniform
from whale_googleanalytics.server.helpers.plotly import Plotly

from starlette.config import Config

config = Config(".env")


today = datetime.today()
todayStr = today.strftime("%Y%m%d%H%M")

plot_plotly = Plotly()


def to_save_csv(VIEW_ID, df: object):
    """Parses and prints the Analytics Reporting API V4 response.
    Args:
        Pandas dataframe
    Returns:
        None
    """
    random = str(randrange(0, 10))

    df["viewID"] = VIEW_ID
    df.to_csv(random + "_" + todayStr + "_" + VIEW_ID + ".csv", index=False)


def credentials(VIEW_ID, api):
    """Test API connection
    Args:
        Google Analytics view, api type (ga or mcf)
    Returns:
        API call
    """
    print(config("GOOGLE_APPLICATION_CREDENTIALS"))

    SCOPES = ["https://www.googleapis.com/auth/analytics.readonly"]
    KEY_FILE_LOCATION = config("GOOGLE_APPLICATION_CREDENTIALS") + VIEW_ID + ".json"

    credentials = ServiceAccountCredentials.from_json_keyfile_name(
        KEY_FILE_LOCATION, SCOPES
    )

    if api == "ga":
        analytics = build("analyticsreporting", "v4", credentials=credentials)
        return analytics
    elif api == "mcf":
        analytics = build(
            "analytics", "v3", credentials=credentials, cache_discovery=False
        )
        return analytics
    else:
        raise ValueError(
            f"Could not detect API from metric {api}. The metric must start with `ga:` or `mcf:`."
        )


def convert_to_dataframe_ga(response: object):
    """Parses and prints the Analytics Reporting API V4 response.
    Args:
        response: An Analytics Reporting API V4 response.
    Returns:
        Pandas DataFrame
    """
    dataframe = []

    for report in response.get("reports", []):

        columnHeader = report.get("columnHeader", {})
        dimensionHeaders = columnHeader.get("dimensions", [])
        metricHeaders = [
            i.get("name", {})
            for i in columnHeader.get("metricHeader", {}).get("metricHeaderEntries", [])
        ]

        for row in report.get("data", {}).get("rows", []):
            dimensions = row.get("dimensions", [])
            metrics = row.get("metrics", [])[0].get("values", {})

            dict = {}

            for header, dimension in zip(dimensionHeaders, dimensions):
                dict[header] = dimension

            for metricHeader, metric in zip(metricHeaders, metrics):
                dict[metricHeader] = metric

            dataframe.append(dict)

    dataFrame_df = pd.DataFrame(dataframe)

    return dataFrame_df


def convert_to_dataframe_mcf(response, write_header: bool = True):
    """Parses and prints the Analytics Reporting API V3 MCF response.
    Args:
        response: An MCF Reporting API V3 response.
    Returns:
        Pandas DataFrame
    """

    columnHeaders = response.get("columnHeaders", [])

    # write header
    headerRow = []
    for column_header in columnHeaders:
        headerRow.append(column_header["name"])

    dataFrame_df = pd.DataFrame(columns=headerRow)

    # write rows
    row_index = 0
    for raw_row in response.get("rows", []):
        row_single = []
        for cell in raw_row:
            row_single.append(cell["primitiveValue"])
        row_index += 1

        # print(row_index)
        # print(row_single)
        dataFrame_df.loc[row_index] = row_single

    return dataFrame_df


def get_ga_single(
    id,
    start_date,
    end_date,
    metrics_list,
    dimensions_list,
    secondary_metrics,
    analytics: object,
):
    """Queries the Analytics Reporting API V4.
    Args:
        analytics: An authorized Analytics Reporting API V4 service object.
    Returns:
        The Analytics Reporting API V4 response.
    """
    metric_list_all = [{"expression": i} for i in metrics_list]

    if secondary_metrics:
        metric_list_all.append({"expression": secondary_metrics})

    return (
        analytics.reports()
        .batchGet(
            body={
                "reportRequests": [
                    {
                        "viewId": id,
                        # "filtersExpression": "ga:eventCategory==Opensite,ga:eventCategory==OutboundLink,ga:eventCategory==Outbound Click",
                        # "dimensionFilterClauses": dimensionFilterClauses,
                        # "samplingLevel": "LARGE",
                        # "pageSize": num_results,
                        "dateRanges": [{"startDate": start_date, "endDate": end_date}],
                        "metrics": metric_list_all,
                        "dimensions": [{"name": j} for j in dimensions_list],
                    }
                ]
            }
        )
        .execute()
    )


def get_mcf_single(
    ids: str, start_date: str, end_date: str, metrics: str, dimensions: str
):
    """Parses and the Multi Channel Funnel V3 API
    Args:
        MCF/GA view, start date, end date, metrics (obj), dimensions (obj)
    Returns:
        response: An MCF API V3 response.
    """
    # print(analytics.__dict__)
    analytics_v3 = credentials(ids, "mcf")
    return (
        analytics_v3.data()
        .mcf()
        .get(
            ids=f"ga:{ids}",
            start_date=start_date,
            end_date=end_date,
            metrics=",".join(metrics),
            dimensions=dimensions,
        )
        .execute()
    )


async def get_ga_analytics(
    VIEW_ID: str,
    start_date: str,
    end_date: str,
    metrics_list: object,
    dimensions_list: object,
    secondary_metrics: object,
):
    """Parses and the Analytics V4 API
    Args:
        MCF/GA view, start date, end date, metrics (obj), dimensions (obj), secondary axis if needed
    Returns:
        response: An GA API V4 response.
    """
    if VIEW_ID:
        global view_id
        view_id = VIEW_ID
    else:
        raise ValueError(
            f"Could not detect API from view {VIEW_ID}. The metric must start with Google Analytics GA or MCF."
        )

    data = get_ga_single(
        view_id,
        start_date,
        end_date,
        metrics_list,
        dimensions_list,
        secondary_metrics,
        credentials(view_id, "ga"),
    )

    data_df = convert_to_dataframe_ga(data)
    to_save_csv(view_id, data_df)

    fig1 = plot_plotly.barchart_1d(
        data_df, metrics_list, dimensions_list, secondary_metrics
    )

    return {"response": data_df, "fig": fig1}


async def get_mcf_analytics(
    ids: str, start_date: str, end_date: str, metrics: str, dimensions: str
):
    """Dataframe and fig for Analytics V3 API
    Args:
        MCF/GA view, start date, end date, metrics (obj), dimensions (obj), secondary axis if needed
    Returns:
        response: single report dataframe and corresponding chart and it saves the DF as a csv
    """
    data = get_mcf_single(ids, start_date, end_date, metrics, dimensions)

    data_df = convert_to_dataframe_mcf(data, write_header=True)
    to_save_csv(ids, data_df)

    fig1 = plot_plotly.barchart_1d(data_df, metrics, dimensions)

    return {"response": data_df, "fig": fig1}


async def plotly_generator(*args):
    """Dataframe and fig for Analytics V3 API
    Args:
        the dataframe and fig for each story
    Returns:
        response: string
    """
    plot_plotly.html_report(*args)

    return {"response": "Html generated"}



# gcp_internal.py: Pull Google Analytics from Official API GA4 - @Credit: 4th Whale Marketing 2021
from datetime import timedelta, datetime

from googleapiclient.discovery import build
from oauth2client.service_account import ServiceAccountCredentials

from whale_googleanalytics.server.enum_list import Metrics, Dimensions

from starlette.config import Config

config = Config(".env")

TESTING = config("TESTING", cast=bool, default=False)

if TESTING:
    config("GOOGLE_APPLICATION_CREDENTIALS")


METRICS = Metrics()
DIMENSIONS = Dimensions()

# Report Settings
METRICS_LIST = [METRICS.USERS]
DIMENSIONS_LIST = [DIMENSIONS.DEVICECATEGORY]


def serializer(object) -> dict:
    return {
        "id": str(object["id"]),
        "data": str(object["data"]),
    }


async def online_connection(id: str):
    """Initializes an Analytics Reporting API V4
    Returns:
        An authorized Analytics Reporting API V4 service object
    """
    # API setup

    SCOPES = ["https://www.googleapis.com/auth/analytics.readonly"]
    KEY_FILE_LOCATION = config("GOOGLE_APPLICATION_CREDENTIALS") + id + ".json"

    credentials = ServiceAccountCredentials.from_json_keyfile_name(
        KEY_FILE_LOCATION, SCOPES
    )

    # Build the service object.
    analytics = build("analyticsreporting", "v4", credentials=credentials)

    data = (
        analytics.reports()
        .batchGet(
            body={
                "reportRequests": [
                    {
                        "viewId": id,
                        "dateRanges": [
                            {
                                "startDate": (
                                    datetime.now() - timedelta(days=1)
                                ).strftime("%Y-%m-%d"),
                                "endDate": datetime.today().strftime("%Y-%m-%d"),
                            }
                        ],
                        "metrics": [{"expression": i} for i in METRICS_LIST],
                        "dimensions": [{"name": j} for j in DIMENSIONS_LIST],
                    }
                ]
            }
        )
        .execute()
    )

    return serializer({"id": id, "data": data})



class Metrics:
    USERS = "ga:users"
    NEWUSERS = "ga:newUsers"
    SESSIONS = "ga:sessions"
    PAGEVIEWS = "ga:pageviews"
    BOUNCERATE = "ga:bounceRate"
    GOAL_1 = "ga:goal1ConversionRate"
    GOAL_2 = "ga:goal2ConversionRate"
    GOAL_3 = "ga:goal3ConversionRate"


class Dimensions:
    DEVICECATEGORY = "ga:deviceCategory"
    NEWUSERS = "ga:newUsers"
    SESSIONS = "ga:sessions"
    PAGEVIEWS = "ga:pageviews"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['whale_googleanalytics',
 'whale_googleanalytics.server',
 'whale_googleanalytics.server.helpers',
 'whale_googleanalytics.server.models',
 'whale_googleanalytics.server.routes']

package_data = \
{'': ['*']}

install_requires = \
['fastapi>=0.70.0,<0.71.0',
 'gino[starlette,pg]>=1.0.1,<2.0.0',
 'google-api-python-client>=2.31.0,<3.0.0',
 'gunicorn>=20.1.0,<21.0.0',
 'oauth2client>=4.1.3,<5.0.0',
 'pandas>=1.3.4,<2.0.0',
 'plotly>=5.4.0,<6.0.0',
 'psycopg2-binary>=2.9.2,<3.0.0',
 'uvicorn>=0.15.0,<0.16.0']

entry_points = \
{'console_scripts': ['start = whale_googleanalytics.main:start']}

setup_kwargs = {
    'name': 'whale-googleanalytics',
    'version': '1.21',
    'description': 'This library is used to fetch data from Google Analytics GA v4 and MCF v3',
    'long_description': None,
    'author': 'PO Boisvert',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
